@@ -59,6 +59,8 @@ enum {
         PROP_X_SESSION,
         PROP_LOCATION,
         PROP_LOGIN_FREQUENCY,
+        PROP_LOGIN_TIME,
+        PROP_LOGIN_HISTORY,
         PROP_ICON_FILE,
         PROP_LOCKED,
         PROP_PASSWORD_MODE,
@@ -90,6 +92,8 @@ struct User {
         gchar        *x_session;
         gchar        *location;
         guint64       login_frequency;
+        gint64        login_time;
+        GVariant     *login_history;
         gchar        *icon_file;
         gchar        *default_icon_file;
         gboolean      locked;
@@ -304,36 +308,42 @@ user_update_from_keyfile (User     *user,
                 /* TODO: validate / normalize */
                 g_free (user->language);
                 user->language = s;
+                g_object_notify (G_OBJECT (user), "language");
         }
 
         s = g_key_file_get_string (keyfile, "User", "XSession", NULL);
         if (s != NULL) {
                 g_free (user->x_session);
                 user->x_session = s;
+                g_object_notify (G_OBJECT (user), "x-session");
         }
 
         s = g_key_file_get_string (keyfile, "User", "Email", NULL);
         if (s != NULL) {
                 g_free (user->email);
                 user->email = s;
+                g_object_notify (G_OBJECT (user), "email");
         }
 
         s = g_key_file_get_string (keyfile, "User", "Location", NULL);
         if (s != NULL) {
                 g_free (user->location);
                 user->location = s;
+                g_object_notify (G_OBJECT (user), "location");
         }
 
         s = g_key_file_get_string (keyfile, "User", "PasswordHint", NULL);
         if (s != NULL) {
                 g_free (user->password_hint);
                 user->password_hint = s;
+                g_object_notify (G_OBJECT (user), "password-hint");
         }
 
         s = g_key_file_get_string (keyfile, "User", "Icon", NULL);
         if (s != NULL) {
                 g_free (user->icon_file);
                 user->icon_file = s;
+                g_object_notify (G_OBJECT (user), "icon-file");
         }
 
         g_object_thaw_notify (G_OBJECT (user));
@@ -386,7 +396,7 @@ save_extra_data (User *user)
         error = NULL;
         data = g_key_file_to_data (keyfile, NULL, &error);
         if (error == NULL) {
-                filename = g_build_filename ("/var/lib/AccountsService/users",
+                filename = g_build_filename (USERDIR,
                                              user->user_name,
                                              NULL);
                 g_file_set_contents (filename, data, -1, &error);
@@ -407,9 +417,9 @@ move_extra_data (const gchar *old_name,
         gchar *old_filename;
         gchar *new_filename;
 
-        old_filename = g_build_filename ("/var/lib/AccountsService/users",
+        old_filename = g_build_filename (USERDIR,
                                          old_name, NULL);
-        new_filename = g_build_filename ("/var/lib/AccountsService/users",
+        new_filename = g_build_filename (USERDIR,
                                          new_name, NULL);
 
         g_rename (old_filename, new_filename);
@@ -1683,6 +1693,18 @@ user_real_get_login_frequency (AccountsUser *user)
         return USER (user)->login_frequency;
 }
 
+static gint64
+user_real_get_login_time (AccountsUser *user)
+{
+        return USER (user)->login_time;
+}
+
+static const GVariant *
+user_real_get_login_history (AccountsUser *user)
+{
+        return USER (user)->login_history;
+}
+
 static const gchar *
 user_real_get_icon_file (AccountsUser *user)
 {
@@ -1770,6 +1792,12 @@ user_set_property (GObject      *object,
         case PROP_LOGIN_FREQUENCY:
                 user->login_frequency = g_value_get_uint64 (value);
                 break;
+        case PROP_LOGIN_TIME:
+                user->login_time = g_value_get_int64 (value);
+                break;
+        case PROP_LOGIN_HISTORY:
+                user->login_history = g_variant_ref (g_value_get_variant (value));
+                break;
         case PROP_AUTOMATIC_LOGIN:
                 user->automatic_login = g_value_get_boolean (value);
                 break;
@@ -1840,6 +1868,12 @@ user_get_property (GObject    *object,
         case PROP_LOGIN_FREQUENCY:
                 g_value_set_uint64 (value, user->login_frequency);
                 break;
+        case PROP_LOGIN_TIME:
+                g_value_set_int64 (value, user->login_time);
+                break;
+        case PROP_LOGIN_HISTORY:
+                g_value_set_variant (value, user->login_history);
+                break;
         case PROP_LOCKED:
                 g_value_set_boolean (value, user->locked);
                 break;
@@ -1900,6 +1934,8 @@ user_accounts_user_iface_init (AccountsUserIface *iface)
         iface->get_xsession = user_real_get_xsession;
         iface->get_location = user_real_get_location;
         iface->get_login_frequency = user_real_get_login_frequency;
+        iface->get_login_time = user_real_get_login_time;
+        iface->get_login_history = user_real_get_login_history;
         iface->get_icon_file = user_real_get_icon_file;
         iface->get_locked = user_real_get_locked;
         iface->get_password_mode = user_real_get_password_mode;
@@ -1929,4 +1965,5 @@ user_init (User *user)
         user->locked = FALSE;
         user->automatic_login = FALSE;
         user->system_account = FALSE;
+        user->login_history = NULL;
 }
